#include "cquad.h"

double cquad_dqawoe(double f(), double a, double b, double omega, int integr, double epsabs, double epsrel, double *abserr, int *neval, int *ier, double *alist, double *blist, double *rlist, double *elist, int *iord, int *last, int *nnlog, double **chebmo)
{
/******************************************************************************
***  Date written:         03/31/2007  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Last revision date:   11/02/2012  (mm/dd/yyyy)                         ***
***-------------------------------------------------------------------------***
***  Author:    Gianluca Russo    (gianlurusso@fastwebnet.it)               ***
***             Salvatore Cuomo   (salvatore.cuomo@unina.it)                ***
***                                                                         ***
***             University of Naples "Federico II"                          ***
***             Dipartimento di Matematica e Applicazioni "R. Caccioppoli"  ***
***									    ***
*** 									    ***
***									    ***
*** This code is provided "as is", without any warranty of any kind,        ***
*** either expressed or implied, including but not limited to, any implied  ***
*** warranty of merchantibility or fitness for any purpose. In no event     ***
*** will any party who distributed the code be liable for damages or for    ***
*** any claim(s) by any other party, including but not limited to, any      ***
*** lost profits, lost monies, lost data or data rendered inaccurate,	    ***
*** losses sustained by third parties, or any other special, incidental or  ***	
*** consequential damages arising out of the use or inability to use the    ***
*** program, even if the possibility of such damages has been advised	    ***	
*** against. The entire risk as to the quality, the performance, and the    ***	
*** fitness of the program for any particular purpose lies with the party   ***
*** using the code.							    ***
***									    ***
*** This code, and any derivative of this code, may not be used in a        ***
*** commercial package without the prior explicit written permission of     ***
*** the authors. Verbatim copies of this code may be made and distributed   ***
*** in any medium, provided that this copyright notice is not removed or    ***
*** altered in any way. No fees may be charged for distribution of the      ***
*** codes, other than a fee to cover the cost of the media and a            ***
*** reasonable handling fee.						    ***
***-------------------------------------------------------------------------***
***  Purpose:                                                               ***
***                                                                         ***
***    The function provides a C interface to the fortran routine dqawoe    ***
***    of quadpack package.                                                 ***
***                                                                         ***
***    The function calculates an approximation result to a given definite  ***
***    integral i = integral of f(x) * w(x) over (a,b), where:              ***
***                                                                         ***
***                    w(x) = cos(omega * x)   or                           ***
***                    w(x) = sin(omega * x)                                ***
***                                                                         ***
***    hopefully satisfying following claim for accuracy:                   ***
***                                                                         ***
***              abs(i - result) <= max(epsabs, epsrel * abs(i))            ***
***-------------------------------------------------------------------------***
***  Description:                                                           ***
***                                                                         ***
***    A routine for the integration of COS(omega * x) * f(x) or            ***
***    SIN(omega * x) * f(x) over a finite interval (a,b).                  ***
***    omega is specified by the user.                                      ***
***    The rule evaluation component is based on the modified               ***
***    Clenshaw-Curtis technique. An adaptive subdivision scheme is used    ***
***    connected with an extrapolation procedure, which is a modification   ***
***    of that in cquad_dqagse and provides the possibility to deal even    ***
***    with singularities in f.                                             ***
***                                                                         ***
***    Expert user version (provide more information and control than the   ***
***    easier version cquad_dqwo).                                          ***
***                                                                         ***
***    Double precision version.                                            ***
***-------------------------------------------------------------------------***
***  Input:                                                                 ***
***                                                                         ***
***    f        function subprogram defining the integrand function f(x)    ***
***                                                                         ***
***    a        lower limit of integration                                  ***
***                                                                         ***
***    b        upper limit of integration                                  ***
***                                                                         ***
***    omega    parameter in the integrand weight function                  ***
***                                                                         ***
***    integr   indicates which of the weight functions is used:            ***
***                                                                         ***
***                 integr = 1 -->     w(x) = cos(omega * x)                ***
***                 integr = 2 -->     w(x) = sin(omega * x)                ***
***                                                                         ***
***    epsabs   absolute accoracy requested                                 ***
***                                                                         ***
***    epsrel   relative accoracy requested                                 ***
***-------------------------------------------------------------------------***
***  Output:                                                                ***
***                                                                         ***
***    res      approximation to the integral                               ***
***                                                                         ***
***    abserr   estimate of the modulus of the absolute error               ***
***                                                                         ***
***    neval    number of integrand evaluations                             ***
***                                                                         ***
***    ier      normal termination flag:                                    ***
***                                                                         ***
***               ier = 0 --> normal and reliable termination of the        ***
***                           function. It is assumed that the requested    ***
***                           accuracy has been achieved.                   ***
***                                                                         ***
***               ier = 1 --> maximum number of subdivisions allowed has    ***
***                           been achieved.                                ***
***                                                                         ***
***               ier = 2 --> the occurrence of roundoff error is detected  ***
***                           which prevents the requested tolerance from   ***
***                           being achieved.                               ***
***                                                                         ***
***               ier = 3 --> extremely bad integrand behaviour occurs at   ***
***                           some points of the integration interval.      ***
***                                                                         ***
***               ier = 6 --> the input is invalid because (epsabs <= 0 and ***
***                           epsrel < max(50*rel.mach.acc., 0.5d-28)) or   ***
***                           or (integr != 1 and integr != 2) or icall < 1 ***
***                           or maxp1 < 1.                                 ***
***                           In all these events res, abserr, neval, last, ***
***                           rlist(1), elist(1), iord(1) and nnlog(1) are  *** 
***                           set to zero. alist(1) and blist(1) are set to ***
***                           a and b respectively.                         ***
***                                                                         ***
***    alist    vector of dimension at least limit, the first last          ***
***             elements of which are the left end points of the            ***
***             subintervals in the partition of the given integration      ***
***             range (a,b)                                                 *** 
***                                                                         ***
***    blist    vector of dimension at least limit, the first last          ***
***             elements of which are the right end points of the           ***
***             subintervals in the partition of the given integration      ***
***             range (a,b)                                                 ***  
***                                                                         ***
***    rlist    vector of dimension at least limit, the first last          ***
***             elements of which are the integral approximations on the    ***
***             subintervals                                                ***
***                                                                         ***
***    elist    vector of dimension at least limit, the first last          ***
***             elements of which are the moduli of the absolute error      ***
***             estimates on the subintervals                               ***
***                                                                         ***
***    last     number of subintervals produced in the subdivision process  ***
***                                                                         ***
***    nnlog    vector of dimension at least limit, containing the          ***
***             subdivision levels of the subintervals, i.e. iwork(i) = l   ***
***             means that the subinterval numbered i is of length          ***
***             abs(b - a) * 2 * *(1-l)                                     ***
***                                                                         ***
***    momcom   indicating that the chebyshev moments have been computed    ***
***             for intervals of lengths (abs(b - a)) * 2 * *(-l),          ***
***             l = 0, 1, 2, ..., momcom - 1                                ***
***             (momcom < maxp1)                                            ***
***                                                                         ***
***    chebmo   array of dimension (maxp1,25) containing the chebyshev      ***
***             moments                                                     ***
***-------------------------------------------------------------------------***
***  Workspace:                                                             ***
***                                                                         ***
***    limit    gives an upper bound on the number of subintervals in the   ***
***             partition of (a,b) (at least 1)                             ***
***                                                                         ***
***    icall    if cquad_dqawoe is to be used only once, icall must be set  ***
***             to 1.  Assume that during this call, the chebyshev moments  ***
***             (for clenshaw-curtis integration of degree 24) have been    ***
***             computed for intervals of lenghts (abs(b - a)) * 2 * *(-l), ***
***             l = 0, 1, 2, ..., momcom - 1.                               ***
***             If icall > 1 this means that cquad_dqawoe has been called   ***
***             twice or more on intervals of the same length abs(b - a).   ***
***             The chebyshev moments already computed are then re-used in  ***
***             subsequent calls.                                           ***
***                                                                         ***
***    maxp1    gives an upper bound on the number of chebyshev moments     ***
***             which can be stored, i.e. for the intervals of lengths      ***
***             abs(b - a) * 2 * *(-l), l = 0, 1, ..., maxp1 - 2            ***
***             (maxp1 > 0)                                                 ***
***                                                                         ***
***    iord     vector of dimension at least limit, the first k elements    ***
***             of which are pointers to the error estimates over the       ***
***             subintervals, such that elist(iord(1)), ..., elist(iord(k)) ***
***             form a decreasing sequence, with k = last if last <=        ***
***             (limit/2 + 2), and k = limit + 1 - last otherwise           ***
***                                                                         ***
***-------------------------------------------------------------------------***
***  References:      QUADPACK - Piessens, de Doncker, Uberhuber, Kahaner   ***
***                                                                         ***
***                   http://netlib.org                                     ***
******************************************************************************/

    int limit = LIMIT, maxp1 = MAXP1, momcom, icall = 1;
    double res;  
 
    dqawoe_(f, &a, &b, &omega, &integr, &epsabs, &epsrel, &limit, &icall, &maxp1, &res, abserr, neval, ier, last, alist, blist, rlist, elist, iord, nnlog, &momcom, chebmo);

    return res;
}
